@@ -25,6 +25,7 @@
    work.  */
 
 #include <config.h>
+#include <stdlib.h>
 #include <glib.h>
 
 #include "gtkhtml-private.h"
@@ -211,7 +212,7 @@ forward (HTMLCursor *cursor)
 	if (!html_object_cursor_forward (cursor->object, cursor)) {
 		HTMLObject *next;
 
-		next = html_object_next_cursor (cursor->object, &cursor->offset);
+		next = html_object_next_cursor (cursor->object, (gint *) &cursor->offset);
 		if (next) {
 			if (!html_object_is_container (next))
 				cursor->offset = (next->parent == cursor->object->parent) ? 1 : 0;
@@ -253,7 +254,7 @@ backward (HTMLCursor *cursor)
 	if (!html_object_cursor_backward (cursor->object, cursor)) {
 		HTMLObject *prev;
 
-		prev = html_object_prev_cursor (cursor->object, &cursor->offset);
+		prev = html_object_prev_cursor (cursor->object, (gint *) &cursor->offset);
 		if (prev) {
 			if (!html_object_is_container (prev))
 				cursor->offset = html_object_get_length (prev);
@@ -298,7 +299,6 @@ html_cursor_up (HTMLCursor *cursor,
 	gint prev_x, prev_y;
 	gint x, y;
 	gint target_x;
-	gint orig_y;
 	gboolean new_line;
 
 	gtk_html_im_reset (engine->widget);
@@ -330,8 +330,6 @@ html_cursor_up (HTMLCursor *cursor,
 
 	target_x = cursor->target_x;
 
-	orig_y = y;
-
 	new_line = FALSE;
 
 	while (1) {
@@ -416,7 +414,6 @@ html_cursor_down (HTMLCursor *cursor,
 	gint prev_x, prev_y;
 	gint x, y;
 	gint target_x;
-	gint orig_y;
 	gboolean new_line;
 
 	gtk_html_im_reset (engine->widget);
@@ -448,8 +445,6 @@ html_cursor_down (HTMLCursor *cursor,
 
 	target_x = cursor->target_x;
 
-	orig_y = y;
-
 	new_line = FALSE;
 
 	while (1) {
@@ -481,11 +476,11 @@ html_cursor_down (HTMLCursor *cursor,
 			}
 
 			new_line = TRUE;
-			if (cursor->object->parent)
-				dir = html_object_get_direction (cursor->object->parent);
-			else
-				dir = HTML_DIRECTION_LTR;
 		}
+		if (cursor->object->parent)
+			dir = html_object_get_direction (cursor->object->parent);
+		else
+			dir = HTML_DIRECTION_LTR;
 
 		if (dir == HTML_DIRECTION_RTL) {
 			if (new_line && x <= target_x) {
@@ -739,7 +734,7 @@ html_cursor_get_prev_char (const HTMLCursor *cursor)
 gboolean
 html_cursor_beginning_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 {
-	HTMLCursor *copy;
+	HTMLCursor copy;
 	HTMLObject *flow;
 	gboolean rv = FALSE;
 	gint level, new_level;
@@ -754,15 +749,14 @@ html_cursor_beginning_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 
 	while (1) {
 		if (!cursor->offset) {
-			copy = html_cursor_dup (cursor);
+			html_cursor_copy (&copy, cursor);
 			if (backward (cursor)) {
 				new_level = html_object_get_parent_level (cursor->object);
 				if (new_level < level
 				    || (new_level == level && flow != cursor->object->parent)) {
-					html_cursor_copy (cursor, copy);
+					html_cursor_copy (cursor, &copy);
 					break;
 				}
-				html_cursor_destroy (copy);
 			} else
 				break;
 		}
@@ -778,7 +772,7 @@ html_cursor_beginning_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 gboolean
 html_cursor_end_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 {
-	HTMLCursor *copy;
+	HTMLCursor copy;
 	HTMLObject *flow;
 	gboolean rv = FALSE;
 	gint level, new_level;
@@ -793,15 +787,14 @@ html_cursor_end_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 
 	while (1) {
 		if (cursor->offset == html_object_get_length (cursor->object)) {
-			copy = html_cursor_dup (cursor);
+			html_cursor_copy (&copy, cursor);
 			if (forward (cursor)) {
 				new_level = html_object_get_parent_level (cursor->object);
 				if (new_level < level
 				    || (new_level == level && flow != cursor->object->parent)) {
-					html_cursor_copy (cursor, copy);
+					html_cursor_copy (cursor, &copy);
 					break;
 				}
-				html_cursor_destroy (copy);
 			} else
 				break;
 		}
@@ -859,7 +852,7 @@ move_to_next_object (HTMLCursor *cursor, HTMLEngine *e)
 {
 	HTMLObject *next;
 
-	next = html_object_next_cursor (cursor->object, &cursor->offset);
+	next = html_object_next_cursor (cursor->object, (gint *) &cursor->offset);
 	if (next && next->parent) {
 		cursor->object = next;
 		cursor->position ++;
@@ -883,7 +876,7 @@ move_to_prev_object (HTMLCursor *cursor, HTMLEngine *e)
 {
 	HTMLObject *prev;
 
-	prev = html_object_prev_cursor (cursor->object, &cursor->offset);
+	prev = html_object_prev_cursor (cursor->object, (gint *) &cursor->offset);
 	if (prev && prev->parent) {
 		cursor->object = prev;
 		cursor->position --;
@@ -1009,24 +1002,23 @@ move_right (HTMLCursor *cursor, HTMLEngine *e)
 
 	retval = TRUE;
 	if (!html_object_cursor_right (cursor->object, e->painter, cursor)) {
-		if (cursor->object->parent) {
-			gboolean rv;
-			HTMLObject *orig = cursor->object;
+		gboolean rv;
+		HTMLObject *orig = cursor->object;
 
-			if (html_object_get_direction (cursor->object->parent) == HTML_DIRECTION_RTL)
-				rv = move_to_prev_object (cursor, e);
+		if (cursor->object->parent && 
+			html_object_get_direction (cursor->object->parent) == HTML_DIRECTION_RTL)
+			rv = move_to_prev_object (cursor, e);
+		else
+			rv = move_to_next_object (cursor, e);
+
+		if (rv && !html_object_is_container (cursor->object) && cursor->object->parent == orig->parent) {
+			if (html_object_get_direction (cursor->object) == HTML_DIRECTION_RTL)
+				cursor->offset --;
 			else
-				rv = move_to_next_object (cursor, e);
-
-			if (rv && !html_object_is_container (cursor->object) && cursor->object->parent == orig->parent) {
-				if (html_object_get_direction (cursor->object) == HTML_DIRECTION_RTL)
-					cursor->offset --;
-				else
-					cursor->offset ++;
-			}
-
-			return rv;
+				cursor->offset ++;
 		}
+
+		return rv;
 	}
 	return retval;
 }
